unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ImgList, ToolWin, ComCtrls, ExtCtrls, StdCtrls, Spin, Buttons;

type
  TControlDragKind = (dkNone, dkTopLeft, dkTop, dkTopRight, dkRight,
    dkBottomRight, dkBottom, dkBottomLeft, dkLeft, dkClient);

type
  TDrawStyle = (dsClear, dsEllipse, dsLine, dsRectangle, dsRoundRect,
    dsRenderLine, dsText, dsFloodFill);

type
  TfrmImgBmpEdit = class(TForm)
    tbImgBmp: TToolBar;
    ImageList1: TImageList;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    tbtnFloodFill: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    tbFont: TToolBar;
    cbFont: TComboBox;
    ToolButton10: TToolButton;
    tbtnBold: TToolButton;
    tbtnItalic: TToolButton;
    tbtnUnderline: TToolButton;
    ToolButton14: TToolButton;
    gbPenSetting: TGroupBox;
    lblPenWidth: TLabel;
    sePenWidth: TSpinEdit;
    lblPenStyle: TLabel;
    cbPenStyle: TComboBox;
    Panel1: TPanel;
    RichEdit1: TRichEdit;
    Image1: TImage;
    cbColor: TComboBox;
    ToolButton3: TToolButton;
    cbFontSize: TComboBox;
    ToolButton7: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure cbPenColorDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure cbPenStyleDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ToolButton1Click(Sender: TObject);
    procedure tbtnBoldClick(Sender: TObject);
    procedure tbtnItalicClick(Sender: TObject);
    procedure tbtnUnderlineClick(Sender: TObject);
    procedure cbFontChange(Sender: TObject);
    procedure cbFontSizeChange(Sender: TObject);
    procedure sePenWidthChange(Sender: TObject);
    procedure RichEdit1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RichEdit1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure RichEdit1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure cbFontColorChange(Sender: TObject);
    procedure cbColorChange(Sender: TObject);
  private
    FContentChanged: Boolean;
    FDrawStyle: TDrawStyle;
    FBeginPoint: TPoint;
    FRendering: Boolean;
    FBold: Boolean; // r
    FItalic: Boolean; // r
    FUnderline: Boolean; // [u

    FDownPos: TPoint; { position of last mouse down, screen-relative }
    FDragKind: TControlDragKind; { kind of drag in progress }
    function GetDragging: Boolean;

    procedure DrawLine(x1, y1, x2, y2: integer);
    procedure DrawEllipse(x1, y1, x2, y2: integer);
    procedure DrawRectangle(x1, y1, x2, y2: integer);
    procedure DrawRoundRect(x1, y1, x2, y2, x3, y3: integer);
    function GetPenStyle(AIndex: integer): TPenStyle;
    procedure PaintRichEditToImgBmp;
    procedure ShowRichEdit(X, Y: integer);
  public
    property DraggingControl: Boolean read GetDragging;
  end;

var
  frmImgBmpEdit: TfrmImgBmpEdit;

implementation

{$R *.DFM}
Const
  { Set of cursors to use while moving over and dragging on controls. }
  DragCursors : Array [TControlDragKind] of TCursor =
    (crDefault, crSizeNWSE, crSizeNS, crSizeNESW, crSizeWE,
      crSizeNWSE, crSizeNS, crSizeNESW, crSizeWE, crHandPoint);

  { Width of "hot zone" for dragging around the control borders. }
  HittestMargin = 3;

Type
  TCracker = Class(TControl); { Needed since TControl.MouseCapture is protected }

procedure TfrmImgBmpEdit.FormCreate(Sender: TObject);
begin
  FDrawStyle := dsLine;

  Image1.Canvas.Brush.Color := clBlack;
  Image1.Canvas.FillRect(Image1.ClientRect);

  cbPenStyle.ItemIndex := 0;
  cbColor.Items.Add(IntToStr(clBlack));
  cbColor.Items.Add(IntToStr(clAqua));
  cbColor.Items.Add(IntToStr(clBlue));
  cbColor.Items.Add(IntToStr(clDkGray));
  cbColor.Items.Add(IntToStr(clFuchsia));
  cbColor.Items.Add(IntToStr(clGray));
  cbColor.Items.Add(IntToStr(clGreen));
  cbColor.Items.Add(IntToStr(clInfoBk));
  cbColor.Items.Add(IntToStr(clLime));
  cbColor.Items.Add(IntToStr(clMaroon));
  cbColor.Items.Add(IntToStr(clNavy));
  cbColor.Items.Add(IntToStr(clOlive));
  cbColor.Items.Add(IntToStr(clPurple));
  cbColor.Items.Add(IntToStr(clRed));
  cbColor.Items.Add(IntToStr(clSilver));
  cbColor.Items.Add(IntToStr(clTeal));
  cbColor.Items.Add(IntToStr(clYellow));
  cbColor.Items.Add(IntToStr(clWhite));
  cbColor.ItemIndex := 0;

  cbFont.Items.Assign(Screen.Fonts);
  cbFont.ItemIndex := 0;

  cbFontSize.ItemIndex := 0;
  cbFont.ItemIndex := 0;
end;

procedure TfrmImgBmpEdit.cbPenColorDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var c: TCanvas;
begin
  c := (Control as TComboBox).Canvas;
  c.Brush.Color := clWhite;
  c.FillRect(Rect);
  InflateRect(Rect, -1, -1);
  c.Brush.Color := StrToInt((Control as TComboBox).Items[Index]);
  c.FillRect(Rect);
end;

procedure TfrmImgBmpEdit.cbColorChange(Sender: TObject);
begin
  RichEdit1.Font.Color := StrToInt(cbColor.Items[cbColor.ItemIndex]);
  if RichEdit1.Visible then
    RichEdit1.SetFocus;
end;

procedure TfrmImgBmpEdit.cbPenStyleDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var c: TCanvas;
begin
  c := (Control as TComboBox).Canvas;
  c.Brush.Color := clWhite;
  c.FillRect(Rect);

  c.Pen.Style := GetPenStyle(Index);
  c.Pen.Color := clRed;
  c.MoveTo(0, Rect.Top + ((Rect.Bottom - Rect.Top) div 2));
  c.LineTo(Rect.Right, Rect.Top + ((Rect.Bottom - Rect.Top) div 2));
end;

// eu
procedure TfrmImgBmpEdit.DrawLine(x1, y1, x2, y2: integer);
begin
  Image1.Canvas.MoveTo(x1, y1);
  Image1.Canvas.LineTo(x2, y2);
end;

// e
procedure TfrmImgBmpEdit.DrawEllipse(x1, y1, x2, y2: integer);
begin
  Image1.Canvas.Ellipse(x1, y1, x2, y2);
end;

// ex
procedure TfrmImgBmpEdit.DrawRectangle(x1, y1, x2, y2: integer);
begin
  Image1.Canvas.Rectangle(x1, y1, x2, y2);
end;

// e
procedure TfrmImgBmpEdit.DrawRoundRect(x1, y1, x2, y2, x3, y3: integer);
begin
  Image1.Canvas.RoundRect(x1, y1, x2, y2, x3, y3);
end;

procedure TfrmImgBmpEdit.Image1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var tmpPixel: TColor;
begin
  FContentChanged := True;
  if FDrawStyle = dsFloodFill then begin
    tmpPixel := Image1.Canvas.Pixels[X, Y];
    Image1.Canvas.Brush.Color := StrToInt(cbColor.Items[cbColor.ItemIndex]);
    Image1.Canvas.FloodFill(X, Y, tmpPixel, fsSurface);
  end else begin
    Image1.Canvas.Pen.Color := StrToInt(cbColor.Items[cbColor.ItemIndex]);
    Image1.Canvas.Pen.Style := GetPenStyle(cbPenStyle.ItemIndex);
    Image1.Canvas.Pen.Width := sePenWidth.Value;
  end;

  if FDrawStyle = dsText then
    ShowRichEdit(X, Y);

  if FDrawStyle in [dsClear, dsRenderLine] then // BNu
    FRendering := True;

  Image1.Canvas.MoveTo(X, Y);
  FBeginPoint := Point(X, Y);
end;

procedure TfrmImgBmpEdit.Image1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if FRendering then begin
    Image1.Canvas.MoveTo(FBeginPoint.x, FBeginPoint.y);
    FBeginPoint := Point(X, Y);
    Image1.Canvas.LineTo(X, Y);
  end;
end;

procedure TfrmImgBmpEdit.Image1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  case FDrawStyle of
    dsLine: DrawLine(FBeginPoint.x, FBeginPoint.y, X, Y);
    dsEllipse: DrawEllipse(FBeginPoint.x, FBeginPoint.y, X, Y);
    dsRectangle: DrawRectangle(FBeginPoint.x, FBeginPoint.y, X, Y);
    dsRoundRect: DrawRoundRect(FBeginPoint.x,
                               FBeginPoint.y,
                               X,
                               Y,
                               (FBeginPoint.x - X) div 2,
                               (FBeginPoint.y - Y) div 2);
  end;
  if FRendering then
    FRendering := False;
end;

function TfrmImgBmpEdit.GetPenStyle(AIndex: integer): TPenStyle;
begin
  case AIndex Of
    0: Result := psSolid; // psSolid
    1: Result := psDash; // psDash
    2: Result := psDot; // psDot
    3: Result := psDashDot; // psDashDot
    4: Result := psDashDotDot; // psDashDotDot
  else
    Result := psSolid;
  end;
end;

procedure TfrmImgBmpEdit.PaintRichEditToImgBmp;
var i, iTxtHeight, xPos, yPos: integer;
begin
  Image1.Canvas.Font.Assign(RichEdit1.Font);

  xPos := RichEdit1.Left;
  yPos := RichEdit1.Top;
  for i := 0 to RichEdit1.Lines.Count -1 do begin
    Image1.Canvas.TextOut(xPos, yPos, RichEdit1.Lines[i]);
    iTxtHeight := Image1.Canvas.TextHeight(RichEdit1.Lines[i]);
    yPos := yPos + iTxtHeight;
  end;
end;

procedure TfrmImgBmpEdit.ShowRichEdit;
begin
  if RichEdit1.Visible then begin
    RichEdit1.Visible := False;
    PaintRichEditToImgBmp;
    tbFont.Visible := False;
  end else begin
    RichEdit1.Left := X;
    RichEdit1.Top := Y;
    RichEdit1.Visible := True;
    tbFont.Visible := True;
  end;
  RichEdit1.Clear;
end;

procedure TfrmImgBmpEdit.ToolButton1Click(Sender: TObject);
var i: integer;
begin
  for i := 0 to tbImgBmp.ControlCount -1 do begin
    if tbImgBmp.Controls[i] is TToolButton then
      (tbImgBmp.Controls[i] as TToolButton).Down := False;
  end;
  (Sender as TToolButton).Down := True;

  case (Sender as TToolButton).Tag of
    1: FDrawStyle := dsLine; // u
    2: FDrawStyle := dsRectangle; // ex
    3: FDrawStyle := dsRoundRect; // 
    4: FDrawStyle := dsEllipse; // 
    5: FDrawStyle := dsRenderLine; // Nu
    6: FDrawStyle := dsText; // r
    7: FDrawStyle := dsClear; // 
    8: FDrawStyle := dsFloodFill; // 
  end;

  if ((Sender as TToolButton).Tag = 6) then // r
    gbPenSetting.Visible := False
  else
    gbPenSetting.Visible := True;
  tbFont.Visible := (Sender as TToolButton).Tag = 6;
  if RichEdit1.Visible then
    RichEdit1.Visible := (Sender as TToolButton).Tag = 6;
end;

procedure TfrmImgBmpEdit.tbtnBoldClick(Sender: TObject);
begin
  FBold := not FBold;
  tbtnBold.Down := FBold;
  if FBold then
    RichEdit1.Font.Style := RichEdit1.Font.Style + [fsBold]
  else
    RichEdit1.Font.Style := RichEdit1.Font.Style - [fsBold];
  RichEdit1.SetFocus;
end;

procedure TfrmImgBmpEdit.tbtnItalicClick(Sender: TObject);
begin
  FItalic := not FItalic;
  tbtnItalic.Down := FItalic;
  if FItalic then
    RichEdit1.Font.Style := RichEdit1.Font.Style + [fsItalic]
  else
    RichEdit1.Font.Style := RichEdit1.Font.Style - [fsItalic];
  RichEdit1.SetFocus;
end;

procedure TfrmImgBmpEdit.tbtnUnderlineClick(Sender: TObject);
begin
  FUnderline := not FUnderline;
  tbtnUnderline.Down := FUnderline;
  if FUnderline then
    RichEdit1.Font.Style := RichEdit1.Font.Style + [fsUnderline]
  else
    RichEdit1.Font.Style := RichEdit1.Font.Style - [fsUnderline];
  RichEdit1.SetFocus;
end;

procedure TfrmImgBmpEdit.cbFontChange(Sender: TObject);
begin
  RichEdit1.Font.Name := cbFont.Text;
  RichEdit1.SetFocus;
end;

procedure TfrmImgBmpEdit.cbFontSizeChange(Sender: TObject);
begin
  try
    RichEdit1.Font.Size := StrToInt(cbFontSize.Text);
  except
    RichEdit1.Font.Size := 12;
  end;
  RichEdit1.SetFocus;
end;

procedure TfrmImgBmpEdit.sePenWidthChange(Sender: TObject);
begin
  try
    cbPenStyle.Enabled := sePenWidth.Value = 1;
    if not cbPenStyle.Enabled then
      cbPenStyle.ItemIndex := 0;
  except
  end;
end;

procedure TfrmImgBmpEdit.cbFontColorChange(Sender: TObject);
begin
  RichEdit1.Font.Color := StrToInt(cbColor.Items[cbColor.ItemIndex]);
  RichEdit1.SetFocus;
end;

{ Perform hittest on the mouse position.
  Position is in client coordinates for the passed control. }
function GetDragKind(Control: TControl; X, Y: Integer): TControlDragKind;
var R: TRect;
begin
  R := Control.ClientRect;
  Result := dkNone;
  if Abs(X - R.Left) <= HittestMargin then
    if Abs(Y - R.Top) <= HittestMargin then
      Result := dkTopLeft
    else if Abs(Y - R.Bottom) <= HittestMargin then
      Result := dkBottomLeft
    else
      Result := dkLeft
  else if Abs(X-R.Right) <= HittestMargin then
    if Abs(Y - R.Top) <= HittestMargin then
      Result := dkTopRight
    else if Abs(Y - R.Bottom) <= HittestMargin then
      Result := dkBottomRight
    else
      Result := dkRight
  else if Abs(Y - R.Top) <= HittestMargin then
    Result := dkTop
  else if Abs(Y - R.Bottom) <= HittestMargin then
    Result := dkBottom
  else if PtInRect(R, Point(X,Y)) then
    Result := dkClient;
end; { GetDragKind }

procedure TfrmImgBmpEdit.RichEdit1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and (Shift = [ssLeft]) then Begin
    FDragKind := GetDragKind(RichEdit1, X, Y);
    if FDragKind <> dkNone then Begin
      With TCracker(RichEdit1) Do Begin
        { Record current position screen-relative, the origin for the
          client-relative position will move if the form is moved or
          resized on Left/Top sides. }
        FDownPos := ClientToScreen(Point(X, Y));
        MouseCapture := True;
      End;
    End;
  End;
end;

procedure TfrmImgBmpEdit.RichEdit1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  dx, dy: Integer;
  pt: TPoint;
  R: TRect;
begin
  RichEdit1.Cursor := DragCursors[GetDragKind(RichEdit1, X, Y)];

  if DraggingControl then
    with RichEdit1 do begin
      pt := ClientToScreen(Point(X, Y));
      dx := pt.X - FDownPos.X;
      dy := pt.Y - FDownPos.Y;
      { Update stored mouse position to current position. }
      FDownPos := pt;
      R := BoundsRect;
      case FDragKind of
        dkTopLeft: begin
                     R.Left := R.Left + dx;
                     R.Top := R.Top + dy;
                   end; { Case dkTopLeft }
        dkTop: R.Top := R.Top + dy; { Case dkTop }
        dkTopRight: begin
                      R.Right := R.Right + dx;
                      R.Top := R.Top + dy;
                    end; { Case dkTopRight }
        dkRight: R.Right := R.Right + dx; { Case dkRight }
        dkBottomRight: begin
                         R.Right := R.Right + dx;
                         R.Bottom := R.Bottom + dy;
                       end; { Case dkBottomRight }
        dkBottom: R.Bottom := R.Bottom + dy; { Case dkBottom }
        dkBottomLeft: begin
                        R.Left := R.Left + dx;
                        R.Bottom := R.Bottom + dy;
                      end; { Case dkBottomLeft }
        dkLeft: R.Left := R.Left + dx; { Case dkLeft }
        dkClient: OffsetRect(R, dx, dy); { Case dkClient }
      end; { Case }
      { Don't let the Control be resized to nothing }
      if ((R.Right - R.Left ) > HittestMargin*2) and
         ((R.Bottom - R.Top ) > HittestMargin*2) then
        Boundsrect := R;
    end; { With }
end;

procedure TfrmImgBmpEdit.RichEdit1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if DraggingControl then begin
    FDragKind := dkNone;
    With TCracker(RichEdit1) do 
      MouseCapture := False;
  end;
end;

{ Read method for ControlDragging property, returns true if form in drag mode }
function TfrmImgBmpEdit.GetDragging: Boolean;
begin
  Result := FDragKind <> dkNone;
end;

end.
